package com.hywa.banktest.bankframework.service;

import java.io.FileNotFoundException;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import com.hywa.banktest.entity.PaymentRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;

import com.hywa.banktest.bankframework.exception.ToManyTradeException;
import com.hywa.banktest.bankframework.pack.Dlmaps;
import com.hywa.banktest.bankframework.pack.TransVo;
import com.hywa.banktest.bankframework.utils.FtpUtil;
import com.hywa.banktest.kit.DateKit;

public abstract class BankService {
	@Autowired
	FtpUtil ftpUtil;
	@Value("${money.clear.ftp.dir}")
	protected String ftpUploadDir;
	

	/**
	 * 当天资金结算次数
	 */
	protected Integer todayMoneyClearCount = 200;
	
	/**
	 * 1.1	对公账户余额查询(8910)，请求银行接口
	 * @return
	 */
	public abstract Dlmaps queryBalance();
	/**
	 * 1.1	个人账户开户校验(8911)，请求银行接口
	 * @param CUST_ID   客户编号
	 * @param CARD_ID   身份证号
	 * @param CARD_TYPE 证件类型02
	 * @param CUST_NAME 客户姓名
	 * @param CUST_ACC  客户银行账号
	 * @param CUST_ACC_TYPE   客户账号类型
	 * @return
	 */
	public abstract Dlmaps personAccountValidate(String CUST_ID, String CARD_ID, String CARD_TYPE, String CUST_NAME, String CUST_ACC, String CUST_ACC_TYPE); 
	
	/**
	 * 1.1	资金清算请求(8912)，请求银行接口
	 * @return
	 * @throws FileNotFoundException 
	 */
	public abstract Dlmaps moneyClearing(List<TransVo> transVos,String companyType);

	/**
	 *资金清算请求，请求银行接口
	 * @param transVos
	 * @return 返回打款不成功的农户信息（农户ID，身份证，银行卡账号，打款失败原因）
	 */
	public abstract List<PaymentRecord> moneyCleared(List<TransVo> transVos);

	/**
	 * 更新订单详细信息，插入打款记录
	 * @param transVos
	 * @return
	 */
	public abstract int updateOrderInfoPaymentRec(List<TransVo> transVos,String fileName,String applyFileName);
	
	/**
	 * 计算交易总金额
	 * @param transVos
	 * @return
	 */
	protected BigDecimal countTotalMoney(List<TransVo> transVos) {
		BigDecimal totalMoney = new BigDecimal("0.00");//交易总金额
		for (int i =0; i < transVos.size(); i++) {
			BigDecimal money = transVos.get(i).getClearingMoneyDetail();
			totalMoney = totalMoney.add(money);
			
			transVos.get(i).setNum(i + 1);//设置序号
			transVos.get(i).setCustomerNum(transVos.get(i).getFarmerPid());
			transVos.get(i).setCardType("1010");
		}
		return totalMoney;
	}
	

	/**
	 * 生成交易文件名
	 * @return
	 */
	protected String generateTransFileName(){
		synchronized(todayMoneyClearCount) {
			String fileName = DateKit.formatDate(new Date(), "yyyyMMdd")+"qut";
			todayMoneyClearCount += 1;
			String newFileName = fileName + String.format("%03d",todayMoneyClearCount) + ".txt";
			
			while (ftpUtil.existFile(ftpUploadDir, newFileName.toString())) {
				todayMoneyClearCount += 1;
				if(todayMoneyClearCount > 999) {
					throw new ToManyTradeException("系统每日限转帐999次");
				}
				newFileName = fileName + String.format("%03d",todayMoneyClearCount) + ".txt";
			}
			return newFileName;
		}
	}
	
	@Scheduled(cron = "0 0 0 * * ?")
    protected void todayMoneyClearCountReset() {
		synchronized(todayMoneyClearCount) {todayMoneyClearCount = 200;}
    }
	
}
