package com.hywa.banktest.bankframework.service;

import java.io.File;
import java.math.BigDecimal;
import java.util.*;

import com.hywa.banktest.entity.ClearingRecord;
import com.hywa.banktest.entity.ClearingRecordDetail;
import com.hywa.banktest.entity.PaymentRecord;
import com.hywa.banktest.service.ClearingRecordDetailService;
import com.hywa.banktest.service.ClearingRecordService;
import com.hywa.banktest.service.OrderInfoService;
import com.hywa.banktest.service.PaymentRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.hywa.banktest.bankframework.pack.Dlmaps;
import com.hywa.banktest.bankframework.pack.IN;
import com.hywa.banktest.bankframework.pack.PUB;
import com.hywa.banktest.bankframework.pack.TransVo;
import com.hywa.banktest.bankframework.socketHandle.BankSocket;
import com.hywa.banktest.bankframework.utils.DateUtils;
import com.hywa.banktest.bankframework.utils.StringUtils;
import com.hywa.banktest.bankframework.utils.TransUtils;
import com.hywa.banktest.bankframework.utils.XmlUtils;
import org.springframework.transaction.annotation.Transactional;


@Component
public class BankServiceImpl extends BankService{

	@Autowired
	BankSocket bankRequestKit;

	@Autowired
	private PaymentRecordService paymentRecordService;

	@Autowired
	private OrderInfoService orderInfoService;

	@Autowired
	private ClearingRecordDetailService clearingRecordDetailService;
	
	@Autowired
	private ClearingRecordService clearingRecordService;

	private static final String TRANS_SUCCESS_CODE="0000";
	
	/**
	 * APP类型 
	 */
	@Value("${dlmaps.pub.app_type}")
	private String app_type;
	/**
	 * 银行代码
	 */
	@Value("${dlmaps.pub.bank_id}")
	private String bank_id;
	/**
	 * 公司代码
	 */
	@Value("${dlmaps.pub.dpt_id}")
	private String dpt_id;
	/**
	 * 单位代码（固定如4000）
	 */
	@Value("${dlmaps.in.company_id}")
	private String company_id;
	/**
	 * 对公账户(发改委账号)
	 */
	@Value("${dlmaps.in.company_acc}")
	private String company_acc;
	/**
	 * 账号类型
	 */
	@Value("${dlmaps.in.company_acc_type}")
	private String company_acc_type;
	/**
	 * 资金清算文件目录
	 */
	@Value("${money.clear.file.base.dir}")
	private String baseDir;

	@Value("${dlmaps.pub.jj.dpt_id}")
	private String jj_dpt_id;
	
	@Override
	public  Dlmaps queryBalance() {
		Dlmaps dlmaps = new Dlmaps();
		
		PUB pub = new PUB();
		pub.setAPP_TYPE(app_type);//APP类型 
		pub.setBANK_ID(bank_id);//银行代码
		pub.setTRADE_ID("8910");//"交易代码"
		pub.setTRAN_TIME(DateUtils.formatDate(new Date(), "yyyyMMddHHmmss"));//时间戳
		pub.setDPT_ID(dpt_id);//公司代码
		pub.setTRADE_SEQ(StringUtils.generateOrderNum());
		dlmaps.setPUB(pub);
		
		IN in = new IN();
		in.setCOMPANY_ID(company_id);//单位代码（固定如4000）
		in.setCOMPANY_ACC(company_acc);//对公账户(发改委账号)
		in.setCOMPANY_ACC_TYPE(company_acc_type);//"账号类型"
		dlmaps.setIN(in);
	
		return bankRequestKit.query(dlmaps);
	}
	
	@Override
	public Dlmaps personAccountValidate(String CUST_ID, String CARD_ID, String CARD_TYPE, String CUST_NAME, String CUST_ACC, String CUST_ACC_TYPE) {
		Dlmaps dlmaps = new Dlmaps();
		
		PUB pub = new PUB();
		pub.setAPP_TYPE(app_type);
		pub.setBANK_ID(bank_id);
		pub.setTRADE_ID("8911");
		pub.setTRAN_TIME(DateUtils.formatDate(new Date(), "yyyyMMddHHmmss"));
		pub.setDPT_ID(dpt_id);
		pub.setTRADE_SEQ(StringUtils.generateOrderNum());
		dlmaps.setPUB(pub);
		
		
		IN in = new IN();
		//客户id
		in.setCUST_ID(CUST_ID);
		//证件号码
		in.setCARD_ID(CARD_ID);
		//证件类型
		in.setCARD_TYE(CARD_TYPE);
		//客户姓名
		in.setCUST_NAME(CUST_NAME);
		//银行账号
		in.setCUST_ACC(CUST_ACC);
		in.setCUST_ACC_TYE(CUST_ACC_TYPE);
		dlmaps.setIN(in);
		return bankRequestKit.query(dlmaps);
	}
	
	@Override
	public Dlmaps moneyClearing(List<TransVo> transVos,String companyType){
		//交易总笔数
		int totalOrderNum = transVos.size();
		BigDecimal totalMoney = countTotalMoney(transVos);
		String newFileName = generateTransFileName();
		TransUtils.transToFile(transVos, totalOrderNum, totalMoney, baseDir, newFileName);
		ftpUtil.uploadFile(ftpUploadDir, newFileName, baseDir + File.separator + newFileName.toString());
		return transPaymentInfo(newFileName,totalOrderNum,totalMoney.toString(),companyType);
	}

	/**
	 * 资金清算请求，请求银行接口
	 *
	 * @param transVos
	 * @return 返回打款不成功的农户信息（农户ID，身份证，银行卡账号，打款失败原因）
	 */
	@Override
	public List<PaymentRecord> moneyCleared(List<TransVo> transVos) {
		List<PaymentRecord> resultList = Collections.EMPTY_LIST;
		if(null!=transVos && transVos.size()>0){
			int totalOrderNum = transVos.size();
			BigDecimal totalMoney = countTotalMoney(transVos);
			String newFileName = "123.txt";//generateTransFileName();
			TransUtils.transToFile(transVos, totalOrderNum, totalMoney, baseDir, newFileName);
			//ftpUtil.uploadFile(ftpUploadDir, newFileName, baseDir + File.separator + newFileName);
			//TODO 发送银行打款申请
			//Dlmaps result =  transPaymentInfo(newFileName,totalOrderNum,totalMoney.toString());
		}
		return resultList;
	}

	/**
	 * 更新订单详细信息，插入打款记录
	 *
	 * @param transVos
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class)
	@Override
	public int updateOrderInfoPaymentRec(List<TransVo> transVos,String fileName,String applyFileName) {
		List<TransVo> successList = new ArrayList<>();
		List<TransVo> failList = new ArrayList<>();
		if(null!=transVos && transVos.size()>0){
			for (TransVo vo: transVos) {
				if(TRANS_SUCCESS_CODE.equals(vo.getBankDealState())){
					vo.setOrderStatus(3);
					vo.setBankReceiptFile(fileName);
					vo.setTransFile(applyFileName);
					successList.add(vo);
				}else{
					vo.setOrderStatus(1);
					failList.add(vo);
				}
			}
			//TODO 处理失败打款记录订单状态
			if(failList.size()>0){
				int j = orderInfoService.updateAllClearingDetailOrderInfo(failList);
				if(j==0){
					orderInfoService.updateNoOrderClearing(failList);
				}
			}
			//TODO 更新成功打款记录订单状态
			if(successList.size()>0) {
				int i =orderInfoService.updateOrderInfoByClearingId(successList);
				if(i==0){orderInfoService.updateNoOrderClearing(failList);}
			}
			//TODO 插入打款记录信息
			paymentRecordService.batchSavePaymentRecordService(transVos);
			//TODO 更新打款记录明细
			clearingRecordDetailService.batchClearingRecordDetailInfo(transVos);
			// TODO 更新打款记录状态
			TransVo transVo = transVos.get(0);
			ClearingRecordDetail recordDetail = clearingRecordDetailService.queryById(transVo.getId());
			if(recordDetail!=null){
				ClearingRecord clearingRecord = new ClearingRecord();
				clearingRecord.setStatus(3);
				clearingRecord.setId(recordDetail.getFkClearingId());
				return clearingRecordService.updateByIdSelective(clearingRecord);
				}
			}
		return 0;
	}

	private Dlmaps transPaymentInfo(String fileName,Integer orderCount,String money,String companyType){
		boolean a = true ;
		if ("1".equals(companyType)){
			a=false;
		}
		Dlmaps requestPac = new Dlmaps();
		PUB pub = new PUB();
		pub.setAPP_TYPE(app_type);
		pub.setBANK_ID(bank_id);
		pub.setTRADE_ID("8912");
		pub.setTRAN_TIME(DateUtils.formatDate(new Date(), "yyyyMMddHHmmss"));
		if (a){
			pub.setDPT_ID(jj_dpt_id);
		}else {
			pub.setDPT_ID(dpt_id);
		}

		pub.setTRADE_SEQ(StringUtils.generateOrderNum());
		requestPac.setPUB(pub);

		IN in = new IN();
		in.setTRADE_FILE(fileName);
		in.setTRADE_NUM(String.valueOf(orderCount));
		in.setTRADE_TOTAL_MONEY(String.valueOf(money));
		in.setTRADE_TIME(DateUtils.formatDate(new Date(), "yyyyMMddHHmmss"));
		requestPac.setIN(in);
		System.err.println(XmlUtils.toXML(requestPac));
		try {
			Dlmaps result =  bankRequestKit.query(requestPac);
			result.setIN(in);
			return result;
		}catch (Exception e) {
			ftpUtil.deleteFile(ftpUploadDir,  fileName);
			throw e;
		}
	}


}
