package com.hywa.banktest.bankframework.utils;

import java.io.*;
import java.math.BigDecimal;
import java.util.ArrayList;
import com.hywa.banktest.bankframework.utils.rsa.Base64;
import java.util.List;

import com.hywa.banktest.bankframework.pack.TransVo;
import com.hywa.banktest.bankframework.utils.rsa.DES;
import com.hywa.banktest.bankframework.utils.string.StringUtil;
import lombok.extern.slf4j.Slf4j;
import sun.misc.BASE64Decoder;

@Slf4j
public class TransUtils {
	private final static String key  = "12345678";

	private final static  String vi = "12345678";
	/**
	 * 
	 * @param transVos   	转帐数据
	 * @param totalOrderNum 总笔数
	 * @param totalMoney 	总金额
	 * @param outPutDir     文件输出目录, 例:C:/dir
	 * @param fileName      输出文件名, 例:hello.txt
	 * @return
	 */
	public static void transToFile(List<TransVo> transVos, int totalOrderNum,BigDecimal totalMoney,String outPutDir, String fileName) {
		StringBuilder ret = new StringBuilder();
		ret.append(totalOrderNum).append("|").append(totalMoney).append("|\n");
		for (TransVo transVo : transVos) {
			ret.append(transVo.getNum()).append("|")
					.append(transVo.getFarmerName()).append("|")
					.append(transVo.getFarmerAccount()).append("|")
					.append(transVo.getCardType()).append("|")
					.append(transVo.getFarmerPid()).append("|")
					.append(transVo.getClearingMoneyDetail()).append("|")
					.append(transVo.getFarmerPid()).append("|")
					.append(transVo.getId()).append("|").append("\n");
		}
		String retStr = ret.toString();

		byte[] bytes = new byte[0];
		try {
			bytes = DES.encryptCBC(retStr.getBytes("GBK"), key.getBytes(), vi.getBytes());
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();

		}
		String encodeToString = Base64.encode(bytes);
		log.info("ret===================>{}",ret);
		writeToFileOut(encodeToString, outPutDir, fileName);
	}


	public static List<TransVo> fileToTransVo(String fileDir, String fileName){
		List<TransVo> transVos = new ArrayList<>();
		String target = fileDir + File.separator + fileName;
		String text = readToLineFile(new File(target), "GBK");
		String lineText = text;
		if (!text.contains("|\n")){
			lineText = desBase64(text);
		}
		String[] lines = lineText.split("\n");
		if (lines != null && lines.length > 1) {
			for (int i = 1; i < lines.length; i++) {
				String[] words = lines[i].split("\\|");
				TransVo transVo1 = new TransVo();
				if (words == null || words.length < 8){
					continue;
				}
				transVo1.setNum(Integer.valueOf(words[0]));
				transVo1.setFarmerName(words[1]);
				transVo1.setFarmerAccount(words[2]);
				transVo1.setCardType(words[3]);
				transVo1.setFarmerPid(words[4]);
				transVo1.setMoney(words[5]);
				transVo1.setCustomerNum(words[6]);
				transVo1.setOrderNo(words[7]);
				if (words.length >= 10) {
					transVo1.setBankDealState(words[8]);
					transVo1.setBankDealMsg(words[9]);
				}
				transVo1.setBankReceiptFile(fileName);
				transVo1.setId(words[7]);
				transVos.add(transVo1);
			}
		}
		return transVos;
	}
	public static String getClearText(String fileDir, String fileName){
		String target = fileDir + File.separator + fileName;
		String lineText = readToFile(new File(target), "GBK");
		String s = desBase64(lineText);
		log.info(s);
		writeToFile(s.replaceAll("\n","\r\n"),fileDir,"text"+fileName);
		return s;
	}

	private static String desBase64(String base){
		if (StringUtil.isEmpty(base))
			return null;
		BASE64Decoder base64Decoder = new BASE64Decoder();
		byte[] bytes = Base64.decode(base);
		byte[] cbc = DES.decryptCBC(bytes, key.getBytes(), vi.getBytes());
		try {
			return new String(cbc,"GBK");
		} catch (UnsupportedEncodingException e) {
			log.error("======={}",e);
			return null;
		}
	}
	/**
	 * 把字符串写入到文件
	 * @param ret   字符串builder
	 * @param outPutDir   文件输出目录, 例:C:/dir
	 * @param fileName    输出文件名, 例:hello.txt
	 */
	private static void writeToFile(StringBuilder ret, String outPutDir, String fileName) {
		FileWriter fw = null;
		try {
			File dir = new File(outPutDir);
			if (!dir.exists()) {
				dir.mkdirs();
			}
			
			String target = outPutDir + File.separator + fileName;
			fw = new FileWriter(target);
			fw.write(ret.toString());
		}
		catch (IOException e) {
			throw new RuntimeException(e);
		}
		finally {
			if (fw != null) {
				try {fw.close();} catch (IOException e) {e.printStackTrace();}
			}
		}
	}

	/**
	 * 把字符串写入到文件
	 * @param ret   字符串builder
	 * @param outPutDir   文件输出目录, 例:C:/dir
	 * @param fileName    输出文件名, 例:hello.txt
	 */
	public static void writeToFile(String ret, String outPutDir, String fileName) {
		FileWriter fw = null;
		try {
			File dir = new File(outPutDir);
			if (!dir.exists()) {
				dir.mkdirs();
			}

			String target = outPutDir + File.separator + fileName;
			fw = new FileWriter(target);
			fw.write(ret);
		}
		catch (IOException e) {
			throw new RuntimeException(e);
		}
		finally {
			if (fw != null) {
				try {fw.close();} catch (IOException e) {e.printStackTrace();}
			}
		}
	}
	/**
	 * 把字符串写入到文件
	 * @param ret   字符串builder
	 * @param outPutDir   文件输出目录, 例:C:/dir
	 * @param fileName    输出文件名, 例:hello.txt
	 */
	public static void writeToFileOut(String ret, String outPutDir, String fileName) {
		OutputStreamWriter fw = null;
		try {
			File dir = new File(outPutDir);
			if (!dir.exists()) {
				dir.mkdirs();
			}

			String target = outPutDir + File.separator + fileName;
			fw = new OutputStreamWriter(new FileOutputStream(target), "GBK");
			fw.write(ret);
		}
		catch (IOException e) {
			throw new RuntimeException(e);
		}
		finally {
			if (fw != null) {
				try {fw.close();} catch (IOException e) {e.printStackTrace();}
			}
		}
	}

	private static String readToFile(File file, String charset) {
		charset = null != charset ? charset : "UTF-8";
		BufferedReader bufferedReader = null;
		if (file.isFile() && file.exists()) {
			try {
				FileInputStream fileInputStream = new FileInputStream(file);
				InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream, charset);
				bufferedReader = new BufferedReader(inputStreamReader);
				StringBuffer sb = new StringBuffer();
				String text = null;
				while ((text = bufferedReader.readLine()) != null) {
					sb.append(text);
				}
				return sb.toString();
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				if (null != bufferedReader) {
					try {
						bufferedReader.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
		}
		return null;
	}
	
	private static String readToLineFile(File file, String charset) {
		charset = null != charset ? charset : "UTF-8";
		BufferedReader bufferedReader = null;
		if (file.isFile() && file.exists()) {
			try {
				FileInputStream fileInputStream = new FileInputStream(file);
				InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream, charset);
				bufferedReader = new BufferedReader(inputStreamReader);
				StringBuffer sb = new StringBuffer();
				String text = null;
				while ((text = bufferedReader.readLine()) != null) {
					sb.append(text).append("\n");
				}
				return sb.toString();
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				if (null != bufferedReader) {
					try {
						bufferedReader.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
		}
		return null;
	}

	//是否base64加密
	private static Boolean isDesBase64(String text){
		String base64Pattern = "^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$";
		Boolean isLegal = text.matches(base64Pattern);
		return isLegal;
	}
	
	public static void main(String args[]) {
//		List<TransVo> transVos = new ArrayList<>();
//		TransVo transVo1 = new TransVo();
//		transVo1.setBankAccount("银行账号1");
//		transVo1.setCardNum("身份证号1");
//		transVo1.setCardType("01（身份证）");
//		transVo1.setCustomerNum("户号1");
//		transVo1.setMoney("100");
//		transVo1.setName("姓名");
//		transVo1.setOrderNo("订单号1");
//		transVos.add(transVo1);
//		TransVo transVo2 = new TransVo();
//		transVo2.setBankAccount("银行账号2");
//		transVo2.setCardNum("身份证号2");
//		transVo2.setCardType("01（身份证）");
//		transVo2.setCustomerNum("户号2");
//		transVo2.setMoney("100");
//		transVo2.setName("姓名1");
//		transVo2.setOrderNo("订单号2");
//		transVos.add(transVo2);
//		transToFile(transVos, 2, 200, "C:\\bank", "hello.txt");
//		List<TransVo> transVo = unTrans("/Users/xubinbin/Desktop/txt", "hello.txt");
//		System.out.println(transVo.get(0).getReturnCode());
		
		System.out.println(readToLineFile(new File("/Users/xubinbin/Desktop/20180226qut008gbk.txt"), "GBK"));
	}
	
}
