package com.hywa.banktest.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hywa.banktest.bankframework.pack.Dlmaps;
import com.hywa.banktest.bankframework.pack.IN;
import com.hywa.banktest.bankframework.pack.PUB;
import com.hywa.banktest.bankframework.pack.TransVo;
import com.hywa.banktest.bankframework.schedule.TransSchedule;
import com.hywa.banktest.bankframework.service.BankServiceImpl;
import com.hywa.banktest.bankframework.socketHandle.BankSocket;
import com.hywa.banktest.bankframework.utils.DateUtils;
import com.hywa.banktest.bankframework.utils.StringUtils;
import com.hywa.banktest.bankframework.utils.TransUtils;
import com.hywa.banktest.common.AjaxJson;
import com.hywa.banktest.common.ResponseEnum;
import com.hywa.banktest.dao.ClearingRecordMapper;
import com.hywa.banktest.entity.CheckBankVO;
import com.hywa.banktest.entity.ClearingRecord;
import com.hywa.banktest.entity.GetBankNo;
import com.hywa.banktest.entity.GetClearText;
import com.hywa.banktest.service.CarbonBankService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class CarbonBankServiceImpl implements CarbonBankService {

    @Autowired
    private BankSocket bankRequestKit;
    @Autowired
    private BankServiceImpl bankService;


    private static final String TRANS_SUCCESS_CODE="0000";

    /**
     * APP类型
     */
    @Value("${dlmaps.pub.app_type}")
    private String app_type;
    /**
     * 银行代码
     */
    @Value("${dlmaps.pub.bank_id}")
    private String bank_id;
    /**
     * 公司代码
     */
    @Value("${dlmaps.pub.dpt_id}")
    private String dpt_id;
    /**
     * 单位代码（固定如4000）
     */
    @Value("${dlmaps.in.company_id}")
    private String company_id;
    /**
     * 对公账户(发改委账号)
     */
    @Value("${dlmaps.in.company_acc}")
    private String company_acc;
    /**
     * 账号类型
     */
    @Value("${dlmaps.in.company_acc_type}")
    private String company_acc_type;

    @Value("${dlmaps.pub.jj.dpt_id}")
    private String jj_dept_id;

    @Value("${dlmaps.in.jj.company_acc}")
    private String jj_company_acc;

    @Override
    public AjaxJson checkBankCard(CheckBankVO checkBankVO) {
        String companyType = checkBankVO.getCompanyType();
        boolean a = true;
        if ("1".equals(companyType)){
            a=false;
        }
        Dlmaps dlmaps = new Dlmaps();

        PUB pub = new PUB();
        pub.setAPP_TYPE(app_type);
        pub.setBANK_ID(bank_id);
        pub.setTRADE_ID("8911");
        pub.setTRAN_TIME(DateUtils.formatDate(new Date(), "yyyyMMddHHmmss"));
        if (a){
            pub.setDPT_ID(jj_dept_id);
        }else {
            pub.setDPT_ID(dpt_id);
        }

        pub.setTRADE_SEQ(StringUtils.generateOrderNum());
        dlmaps.setPUB(pub);
        IN in = new IN();
        //客户id
        in.setCUST_ID(checkBankVO.getFarmerPid());
        //证件号码
        in.setCARD_ID(checkBankVO.getFarmerPid());
        //证件类型
        in.setCARD_TYE("1010");
        //客户姓名
        in.setCUST_NAME(checkBankVO.getFarmerName());
        //银行账号
        in.setCUST_ACC(checkBankVO.getFarmerAccount());
        dlmaps.setIN(in);
        log.info("--------- bankVillate check before ------{}",JSONObject.toJSONString(dlmaps));
        Dlmaps query = bankRequestKit.query(dlmaps);
        log.info("--------- bankVillate check ------{}",JSONObject.toJSONString(query));

        if ("0000".equals(query.getOUT().getRET_CODE())){
            return new AjaxJson(AjaxJson.SUCCESS,AjaxJson.MSGSUCCESS,query.getOUT());

        }
        return new AjaxJson(AjaxJson.FAILURE,query.getOUT().getMSG(),query.getOUT());

    }

    /**
     * 查询对公账户剩余余额
     * @param getBankNo
     * @return
     */
    @Override
    public AjaxJson findCardNumMoney(GetBankNo getBankNo) {
        boolean a = true;
        if ("1".equals(getBankNo.getCompanyType())){
            a=false;
        }
        Dlmaps dlmaps = new Dlmaps();
        //创建公共域
        PUB pub = new PUB();
        pub.setAPP_TYPE(app_type);//APP类型
        pub.setBANK_ID(bank_id);//银行代码
        pub.setTRADE_ID("8910");//"交易代码"
        pub.setTRAN_TIME(DateUtils.formatDate(new Date(), "yyyyMMddHHmmss"));//时间戳
        if (a){
            pub.setDPT_ID(jj_dept_id);//基金会公司代码
        }else {
            pub.setDPT_ID(dpt_id);//发改委公司代码
        }

        pub.setTRADE_SEQ(StringUtils.generateOrderNum());
        dlmaps.setPUB(pub);

        IN in = new IN();
        in.setCOMPANY_ID(company_id);//单位代码（固定如4000）
        if (a){
            in.setCOMPANY_ACC(jj_company_acc);//对公账户(发改委账号)
        }else {
            in.setCOMPANY_ACC(company_acc);//对公账户(发改委账号)
        }

        in.setCOMPANY_ACC_TYPE(company_acc_type);//"账号类型"
        dlmaps.setIN(in);
        Dlmaps query = bankRequestKit.query(dlmaps);
        log.info("------request bankbalance ----------{}", JSONObject.toJSONString(query));
        return new AjaxJson(AjaxJson.SUCCESS,AjaxJson.MSGSUCCESS,query.getOUT());
    }
    @Autowired
    private ClearingRecordMapper clearingRecordMapper;

    @Autowired
    private TransSchedule transSchedule;

    @Override
    public AjaxJson getClearText(GetClearText getClearText) {
        ClearingRecord record = clearingRecordMapper.selectByPrimaryKey(getClearText.getClearId());

        if (record  == null){
            return new AjaxJson(ResponseEnum.NODATA);
        }
        String qutFileName = record.getBankClearingFile();
        String clearText = transSchedule.getClearText(qutFileName);
        if (StringUtils.isEmpty(clearText)){
            return new AjaxJson(ResponseEnum.FAIL);
        }
        return new AjaxJson(clearText);
    }


}
