package com.hywa.banktest.service.impl;

import com.hywa.banktest.bankframework.pack.TransVo;
import com.hywa.banktest.dao.ClearingRecordDetailMapper;
import com.hywa.banktest.entity.ClearingRecordDetail;
import com.hywa.banktest.service.ClearingRecordDetailService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author xiezt
 */
@Slf4j
@Service("clearingRecordDetailService")
public class ClearingRecordDetailServiceImpl implements ClearingRecordDetailService {

    @Resource
    private ClearingRecordDetailMapper clearingRecordDetailMapper;


    /**
     * 通过清款记录id查询请款记录明细
     *
     * @param recordId
     * @return
     */
    @Override
    public List<ClearingRecordDetail> getClearingRecordDetailByRid(String recordId) {
        return clearingRecordDetailMapper.getClearingRecordDetailFkId(recordId);
    }

    /**
     * 更新清款记录明细信息
     *
     * @param detail
     * @return
     */
    @Override
    public int updateClearingRecordDetailInfoByRid(ClearingRecordDetail detail) {
        return clearingRecordDetailMapper.updateClearingRecordDetailInfoByRid(detail);
    }

    /**
     * 批量更新清款订单明细信息
     *
     * @param detailList
     * @return
     */
    @Override
    public int batchClearingRecordDetailInfo(List<TransVo> detailList) {
        List<ClearingRecordDetail> resultList = new ArrayList<>();
        if(null!=detailList && detailList.size()>0){
            for (TransVo tran:detailList) {
                ClearingRecordDetail bean = new ClearingRecordDetail();
                bean.setId(tran.getId());
                bean.setClearingState(tran.getOrderStatus());
                bean.setClearingTimed(new Date());
                bean.setBankReceiptFile(tran.getBankReceiptFile());
                bean.setBankDealState(tran.getBankDealState());
                bean.setBankDealMsg(tran.getBankDealMsg());
                resultList.add(bean);
            }
           return clearingRecordDetailMapper.batchUpdateCleearingRecordDetail(resultList);
        }
        return 0;
    }


    @Override
    public int deleteById(String id) {
        return clearingRecordDetailMapper.deleteByPrimaryKey(id);
    }

    @Override
    public int insertByModel(ClearingRecordDetail record) {
        return clearingRecordDetailMapper.insert(record);
    }

    @Override
    public int insertSelectiveModel(ClearingRecordDetail record) {
        return clearingRecordDetailMapper.insertSelective(record);
    }

    @Override
    public ClearingRecordDetail queryById(String id) {
        return clearingRecordDetailMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByIdSelective(ClearingRecordDetail record) {
        return clearingRecordDetailMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateById(ClearingRecordDetail record) {
        return clearingRecordDetailMapper.updateByPrimaryKey(record);
    }

    @Override
    public List<Map> queryListByParentId(String parentId) {
        //TODO empty implements
        return null;
    }

    @Override
    public List<ClearingRecordDetail> queryModelByName(String name) {
        //TODO empty implements
        return null;
    }
}
